@@ -74,6 +74,19 @@ enum {
         PROP_TRACK_NUMBER,
 };
 
+static int
+is_non_transcoded_resource (GUPnPDIDLLiteResource *resource)
+{
+        GUPnPProtocolInfo *info;
+
+        info = gupnp_didl_lite_resource_get_protocol_info (resource);
+        if (G_UNLIKELY (info == NULL))
+                return -1;
+
+        return gupnp_protocol_info_get_dlna_conversion (info) &
+               GUPNP_DLNA_CONVERSION_TRANSCODED;
+}
+
 static void
 gupnp_didl_lite_object_init (GUPnPDIDLLiteObject *object)
 {
@@ -203,6 +216,18 @@ gupnp_didl_lite_object_get_property (GObject    *object,
                         (value,
                          gupnp_didl_lite_object_get_xml_node (didl_object));
                 break;
+        case PROP_UPNP_NAMESPACE:
+                g_value_set_pointer
+                        (value,
+                         gupnp_didl_lite_object_get_upnp_namespace
+                                (didl_object));
+                break;
+        case PROP_DC_NAMESPACE:
+                g_value_set_pointer
+                        (value,
+                         gupnp_didl_lite_object_get_dc_namespace
+                                (didl_object));
+                break;
         case PROP_ID:
                 g_value_set_string
                         (value,
@@ -284,55 +309,6 @@ gupnp_didl_lite_object_get_property (GObject    *object,
         }
 }
 
-static void
-gupnp_didl_lite_object_lookup_namespaces (GUPnPDIDLLiteObjectPrivate *priv)
-{
-        xmlNs **ns_list;
-
-        if (priv->upnp_ns && priv->dc_ns)
-                return;
-
-        ns_list = xmlGetNsList (priv->xml_doc->doc,
-                                xmlDocGetRootElement (priv->xml_doc->doc));
-
-        if (ns_list) {
-                short i;
-
-                for (i = 0; ns_list[i] != NULL; i++) {
-                        const char *prefix;
-
-                        prefix = (const char *) ns_list[i]->prefix;
-
-                        if (prefix == NULL)
-                                continue;
-
-                        if (! priv->upnp_ns &&
-                            g_ascii_strcasecmp (prefix, "upnp") == 0)
-                                priv->upnp_ns = ns_list[i];
-                        else if (! priv->dc_ns &&
-                                 g_ascii_strcasecmp (prefix, "dc") == 0)
-                                priv->dc_ns = ns_list[i];
-                }
-
-                xmlFree (ns_list);
-        }
-}
-
-static void
-gupnp_didl_lite_object_constructed (GObject *object)
-{
-        GObjectClass               *object_class;
-        GUPnPDIDLLiteObjectPrivate *priv;
-
-        priv = GUPNP_DIDL_LITE_OBJECT (object)->priv;
-
-        gupnp_didl_lite_object_lookup_namespaces (priv);
-
-        object_class = G_OBJECT_CLASS (gupnp_didl_lite_object_parent_class);
-        if (object_class->constructed != NULL)
-                object_class->constructed (object);
-}
-
 static void
 gupnp_didl_lite_object_dispose (GObject *object)
 {
@@ -359,7 +335,6 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
 
         object_class->set_property = gupnp_didl_lite_object_set_property;
         object_class->get_property = gupnp_didl_lite_object_get_property;
-        object_class->constructed = gupnp_didl_lite_object_constructed;
         object_class->dispose = gupnp_didl_lite_object_dispose;
 
         g_type_class_add_private (klass, sizeof (GUPnPDIDLLiteObjectPrivate));
@@ -401,7 +376,6 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
                                       GUPNP_TYPE_XML_DOC,
                                       G_PARAM_WRITABLE |
                                       G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_PRIVATE |
                                       G_PARAM_STATIC_NAME |
                                       G_PARAM_STATIC_NICK |
                                       G_PARAM_STATIC_BLURB));
@@ -409,22 +383,18 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
         /**
          * GUPnPDIDLLiteObject:upnp-namespace
          *
-         * Pointer to the UPNP XML namespace registered with the
-         * XML document containing this object.
+         * Pointer to the UPnP namespace registered with the XML document                * containing this object.
          *
-         * Internal property.
-         *
-         * Stability: Private
          **/
         g_object_class_install_property
                 (object_class,
                  PROP_UPNP_NAMESPACE,
                  g_param_spec_pointer ("upnp-namespace",
                                        "XML namespace",
-                                       "Pointer to the UPNP XML namespace "
+                                       "Pointer to the UPnP XML namespace "
                                        "registered with the XML document "
                                        "containing this object.",
-                                       G_PARAM_WRITABLE |
+                                       G_PARAM_READWRITE |
                                        G_PARAM_CONSTRUCT_ONLY |
                                        G_PARAM_STATIC_NAME |
                                        G_PARAM_STATIC_NICK |
@@ -433,12 +403,9 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
         /**
          * GUPnPDIDLLiteObject:dc-namespace
          *
-         * Pointer to the DublinCore XML namespace registered with
-         * the XML document containing this object.
+         * Pointer to the DublinCore namespace registered with the XML document
+         * containing this object.
          *
-         * Internal property.
-         *
-         * Stability: Private
          **/
         g_object_class_install_property
                 (object_class,
@@ -448,7 +415,7 @@ gupnp_didl_lite_object_class_init (GUPnPDIDLLiteObjectClass *klass)
                                        "Pointer to the Dublin Core XML "
                                        "namespace registered with the XML "
                                        "document containing this object.",
-                                       G_PARAM_WRITABLE |
+                                       G_PARAM_READWRITE |
                                        G_PARAM_CONSTRUCT_ONLY |
                                        G_PARAM_STATIC_NAME |
                                        G_PARAM_STATIC_NICK |
@@ -785,6 +752,8 @@ get_contributor_list_by_name (GUPnPDIDLLiteObject *object,
  * gupnp_didl_lite_object_new_from_xml:
  * @xml_node: The pointer to 'res' node in XML document
  * @xml_doc: The reference to XML document containing this object
+ * @upnp_ns: The pointer to 'upnp' namespace in XML document
+ * @dc_ns: The pointer to 'dc' namespace in XML document
  *
  * Creates a new #GUPnPDIDLLiteObject for the @xml_node.
  *
@@ -798,7 +767,8 @@ gupnp_didl_lite_object_new_from_xml (xmlNode     *xml_node,
 {
         g_return_val_if_fail (xml_node != NULL, NULL);
         g_return_val_if_fail (xml_node->name != NULL, NULL);
-        g_return_val_if_fail (GUPNP_IS_XML_DOC (xml_doc), NULL);
+        g_return_val_if_fail (upnp_ns != NULL, NULL);
+        g_return_val_if_fail (dc_ns != NULL, NULL);
 
         if (g_ascii_strcasecmp ((char *) xml_node->name, "container") == 0)
                 return g_object_new (GUPNP_TYPE_DIDL_LITE_CONTAINER,
@@ -834,6 +804,39 @@ gupnp_didl_lite_object_get_xml_node (GUPnPDIDLLiteObject *object)
         return object->priv->xml_node;
 }
 
+/**
+ * gupnp_didl_lite_object_get_upnp_namespace:
+ * @object: The #GUPnPDIDLLiteObject
+ *
+ * Get the pointer to the UPnP namespace registered with the XML document.
+ *
+ * Return value: The pointer to UPnP namespace in XML document.
+ **/
+xmlNsPtr
+gupnp_didl_lite_object_get_upnp_namespace (GUPnPDIDLLiteObject *object)
+{
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        return object->priv->upnp_ns;
+}
+
+/**
+ * gupnp_didl_lite_object_get_dc_namespace:
+ * @object: The #GUPnPDIDLLiteObject
+ *
+ * Get the pointer to the DublinCore namespace registered with the XML document
+ * containing this object.
+ *
+ * Return value: The pointer to DublinCore namespace in XML document.
+ **/
+xmlNsPtr
+gupnp_didl_lite_object_get_dc_namespace (GUPnPDIDLLiteObject *object)
+{
+        g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
+
+        return object->priv->dc_ns;
+}
+
 /**
  * gupnp_didl_lite_object_get_upnp_class:
  * @object: The #GUPnPDIDLLiteObject
@@ -1280,6 +1283,7 @@ gupnp_didl_lite_object_get_compat_resource
 {
         GUPnPDIDLLiteResource *resource = NULL;
         GList  *resources = NULL;
+        GList  *compat_resources = NULL;
         GList  *res;
 
         g_return_val_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object), NULL);
@@ -1290,15 +1294,32 @@ gupnp_didl_lite_object_get_compat_resource
                 return NULL;
 
         for (res = resources;
-             res != NULL && resource == NULL;
+             res != NULL;
              res = res->next) {
                 resource = (GUPnPDIDLLiteResource *) res->data;
 
-                if (!is_resource_compatible (resource, sink_protocol_info))
-                        resource = NULL;
+                if (is_resource_compatible (resource, sink_protocol_info))
+                        compat_resources = g_list_append (compat_resources,
+                                                          resource);
         }
 
-        if (resource == NULL && lenient)
+        resource = NULL;
+
+        if (compat_resources != NULL) {
+                /* Try to find non-transcoded resource */
+                res = g_list_find_custom (compat_resources,
+                                          NULL,
+                                          (GCompareFunc)
+                                          is_non_transcoded_resource);
+
+                if (res != NULL)
+                        resource = (GUPnPDIDLLiteResource *) res->data;
+                else
+                        /* Just use the first compatible resource */
+                        resource = (GUPnPDIDLLiteResource *)
+                                   compat_resources->data;
+
+        } else if (lenient)
                 /* Just use the first resource */
                 resource = (GUPnPDIDLLiteResource *) resources->data;
 
@@ -1307,6 +1328,7 @@ gupnp_didl_lite_object_get_compat_resource
                 if (res->data != resource)
                         g_object_unref (res->data);
         g_list_free (resources);
+        g_list_free (compat_resources);
 
         return resource;
 }
@@ -1595,7 +1617,7 @@ gupnp_didl_lite_object_set_genre (GUPnPDIDLLiteObject *object,
         g_return_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object));
 
         xml_util_set_child (object->priv->xml_node,
-                            object->priv->dc_ns,
+                            object->priv->upnp_ns,
                             object->priv->xml_doc->doc,
                             "genre",
                             genre);
@@ -1660,14 +1682,26 @@ void
 gupnp_didl_lite_object_set_album_art (GUPnPDIDLLiteObject *object,
                                       const char          *album_art)
 {
+        xmlNode *node;
+        xmlNs *dlna_ns;
+
         g_return_if_fail (object != NULL);
         g_return_if_fail (GUPNP_IS_DIDL_LITE_OBJECT (object));
 
-        xml_util_set_child (object->priv->xml_node,
-                            object->priv->upnp_ns,
-                            object->priv->xml_doc->doc,
-                            "albumArtURI",
-                            album_art);
+        node = xml_util_set_child (object->priv->xml_node,
+                                   object->priv->upnp_ns,
+                                   object->priv->xml_doc->doc,
+                                   "albumArtURI",
+                                   album_art);
+
+        dlna_ns = xmlNewNs (node,
+                            (const unsigned char *)
+                            "urn:schemas-dlna-org:metadata-2-0/",
+                            (const unsigned char *) "dlna");
+        xmlNewNsProp (node,
+                      dlna_ns,
+                      (const unsigned char *) "profileID",
+                      (const unsigned char *) "JPEG_TN");
 
         g_object_notify (G_OBJECT (object), "album-art");
 }
